
library("parallel")


# Parallel computing set up ---------------------------------

# You can ask R how many cores are available, or set this manually
detectCores() - 1

# Set up parallel processing with how many cores to use
cl <- makeCluster(3)

# Tell it how to set up random number generation on each node
clusterSetRNGStream(cl=cl, 64210)

# create some data
dta <- data.frame(group = rep(1:300, each = 10000),
                  norm = rnorm(3000000))

#create a function that you want to run multiple times. This is a silly example of taking the mean of some subgroups

fun1 <- function(x) {
  mean(dta[dta$group == x,"norm"])
}

## Run in serial on one core and see how long it takes

ptm <- proc.time()

output1 <- sapply(1:300,fun1)

proc.time() - ptm

#Run in parallel


#Export objects to all cores
clusterExport(cl,c("dta"))

#create data
ptm <- proc.time()

output2 <-parSapply(cl, 1:300, fun1)

proc.time() - ptm

stopCluster(cl)



######################## 
## More complex example
#########################



# Function to generate some data and analyse it

fun1 <- function(x, corr) {
  dta <- data.frame(village = rep(1:100,each = 10),
                    arm = rep(0:1, each = 500),
                    rnorm_cluster = rep(sqrt(corr)*rnorm(100), each = 10),
                    rnorm_ind = sqrt(1-corr)*rnorm(1000))
  
  dta$outcome <- 1 + 0.5 * dta$arm + dta$rnorm_cluster + dta$rnorm_ind
  
  me_mdl <- lmer(outcome ~  arm + (1 | village) ,
                 data = dta)
  
  c(corr, coef(summary(me_mdl))[2,])
}

# Set up parallel processing with how many cores to use
cl <- makeCluster(3)

# load packages on each core 
clusterEvalQ(cl, library("lme4"))

# Tell it how to set up random number generation on each node
clusterSetRNGStream(cl=cl, 64210)

run <- data.frame()

for(i in c(0.2, 0.5)){
  temp <- parSapply(cl, 1:6, FUN = fun1, corr = i)
  
  run <- rbind(run,
               t(temp))
}
run  

stopCluster(cl)

    
             
######################## 
## Code to run on HPC
######################## 

#Set working directory to HPC space
setwd("//home/lsh304314/rusergroup")


# Function to generate some data and analyse it

fun1 <- function(x, corr) {
  dta <- data.frame(village = rep(1:100,each = 10),
                    arm = rep(0:1, each = 500),
                    rnorm_cluster = rep(sqrt(corr)*rnorm(100), each = 10),
                    rnorm_ind = sqrt(1-corr)*rnorm(1000))
  
  dta$outcome <- 1 + 0.5 * dta$arm + dta$rnorm_cluster + dta$rnorm_ind
  
  me_mdl <- lmer(outcome ~  arm + (1 | village) ,
                 data = dta)
  
  c(corr, coef(summary(me_mdl))[2,])
}

# Set up parallel processing with how many cores to use
## Don't use detectCore() on the HPC
cl <- makeCluster(6)

# load packages on each core 
clusterEvalQ(cl, library("lme4"))

# Tell it how to set up random number generation on each node
clusterSetRNGStream(cl=cl, 64210)

run <- data.frame()

for(i in c(0.2, 0.5)){
  temp <- parSapply(cl, 1:10, FUN = fun1, corr = i)
  
  run <- rbind(run,
               t(temp))
}
run  

stopCluster(cl)

save(run, file = "./rundata.RData")
